#include <vo_stdio.h>
#include <volink.h>     // Linker directives like DLLENTRY
#include <apploader.h>  // RunLibraryFunction etc
#include <string.h>
#include <stdlib.h>
#include <copyfile.h>

#define BUFFER_SIZE 2048

#define MIN(a,b) (((a)<(b))?(a):(b))

u_int16 buffer[2][BUFFER_SIZE];

u_int16 CompareBuffers(register u_int16 *b1, register u_int16 *b2,
		       register u_int16 bytes) {
  u_int16 diffPos = 0;

  /* Quick check */
  if (!memcmp(b1, b2, (bytes+1)/2)) {
    return 0;
  }

  /* Ok, there may be a difference; let's check it out. Do full word
     comparisons. */
  while (bytes > 1) {
    u_int16 diff = *b1++ ^ *b2++;
    if (diff) {
      if (diff & 0xFF00U) {
	return diffPos;
      } else {
	return diffPos+1;
      }
    }
    diffPos += 2;
    bytes -= 2;
  }

  /* Check potential dangling byte */
  if (bytes) {
    if ((*b1++ ^ *b2++) & 0xFF00) {
      return diffPos;
    }
  }

  return 0;
}



DLLENTRY(main)
ioresult main(char *parameters) {
  int nParam = RunLibraryFunction("ParamSpl",ENTRY_MAIN,(u_int16)parameters);
  char *p = parameters, *f1Name=NULL, *f2Name=NULL;
  int i, verbose = 0, f1r, f2r;
  ioresult retVal = S_ERROR;
  FILE *f1=NULL, *f2=NULL;
  int minR;

  for (i=0; i<nParam; i++) {
    if (!strcmp(p, "-h")) {
      printf("Usage: Copy [-h] [f1 f2]\n"
	     "f1\tFile 1\n"
	     "f2\tFile 2\n"
	     "-h\tShow this help\n");
      retVal = S_OK;
      goto finally;
    } else if (!strcmp(p, "-v")) {
      verbose = 1;
    } else {
      if (!f1Name) {
	f1Name = p;
      } else if (!f2Name) {
	f2Name = p;
      } else {
	printf("E: Extraneous parameter \"%s\"\n", p);
	goto finally;
      }
    }
    p += strlen(p)+1;
  }

  if (!f2Name) {
    printf("E: File name(s) missing\n");
    goto finally;
  }

  if (!(f1 = fopen(f1Name, "rb"))) {
    printf("E: Couldn't open \"%s\"\n", f1Name);
    goto finally;
  }

  if (!(f2 = fopen(f2Name, "rb"))) {
    printf("E: Couldn't open \"%s\"\n", f2Name);
    goto finally;
  }

  do {
    u_int32 oldPos = f1->pos;
    u_int16 diffPos;
    f1r = f1->op->Read(f1, buffer[0], 0, BUFFER_SIZE*2);
    f2r = f2->op->Read(f2, buffer[1], 0, BUFFER_SIZE*2);
    minR = MIN(f1r, f2r);
    if ((diffPos = CompareBuffers(buffer[0], buffer[1], minR))) {
      printf("cmp: \"%s\" and \"%s\" differ: byte %ld\n",
	     f1Name, f2Name, oldPos+diffPos+1);
      retVal = S_OK;
      goto finally;
    }
  } while (minR >= BUFFER_SIZE*2);

  if (f1r < f2r) {
    printf("cmp: EOF on \"%s\": byte %ld\n", f1Name, f1->pos);
  }

  if (f2r < f1r) {
    printf("cmp: EOF on \"%s\": byte %ld\n", f2Name, f2->pos);
  }

  retVal = S_OK;
 finally:
  if (f2) {
    fclose(f2);
    f2 = NULL;
  }
  if (f1) {
    fclose(f1);
    f1 = NULL;
  }
  return retVal;
}
