/* For free support for VSIDE, please visit www.vsdsp-forum.com */

#include <vo_stdio.h>
#include <volink.h>     // Linker directives like DLLENTRY
#include <apploader.h>  // RunLibraryFunction etc
#include <consolestate.h>
#include <vo_fat.h>
#include <string.h>
#include <swap.h>
#include <sysmemory.h>
#include <strings.h>
#include "audioinfo.h"


#define NAMELENGTH 255
#define SORT_ELEMENTS 1024

#define LF_SORT_BY_NAME	 1
#define LF_SORT_BY_TIME	 2
#define LF_SORT_BY_SIZE	 4
#define LF_SORT_REVERSE  8
#define LF_AUDIO_ONLY	16
#define LF_FAST		32 /* Only relevant when LF_AUDIO_ONLY selected */
#define LF_SHOWDATE	64
#define LF_SORT_MASK (LF_SORT_BY_NAME|LF_SORT_BY_TIME|LF_SORT_BY_SIZE)

struct SortDirEntry {
  int idx;
  int attr;
  u_int32 size;
  u_int16 date;
  u_int16 time;
  unsigned char shortName[13];
  unsigned char longName[1];
};
struct SortDirEntry *sortDirEntry[SORT_ELEMENTS] = {0};

u_int32 FoundFileSize(VO_FILE *hSearch) {
  DirectoryEntry *de = &hSearch->sectorBuffer[((hSearch->pos-32) >> 1) & 0xff];
  return Swap32Mix(*(u_int32*)(&de->fileSizeLo));
}

u_int32 FoundFileTimeDate(VO_FILE *hSearch) {
  DirectoryEntry *de = &hSearch->sectorBuffer[((hSearch->pos-32) >> 1) & 0xff];
  return Swap32Mix(*(u_int32*)(&de->writeTime));
}


void FreeAllEntries(void) {
  struct SortDirEntry **sde = sortDirEntry;
  while (*sde) {
    free(*sde);
    *sde++ = NULL;
  }
}

int sortFlags = 0;
int SortDirEntryCmp(const void *p1, const void *p2) {
  struct SortDirEntry *sde1 = *((struct SortDirEntry **)p1);
  struct SortDirEntry *sde2 = *((struct SortDirEntry **)p2);
  int res = 0;

  if (sortFlags & LF_SORT_BY_TIME) {
    if (sde1->date == sde2->date) {
      if (sde1->time > sde2->time) {
	res = 1;
      } else if (sde1->time < sde2->time) {
	res = -1;
      }
    } else {
      res = sde1->date-sde2->date;
    }
  } else if (sortFlags & LF_SORT_BY_SIZE) {
    if (sde1->size > sde2->size) {
      res = 1;
    } else if (sde1->size < sde2->size) {
      res = -1;
    }
  } else {
    res = strcasecmp(sde1->longName, sde2->longName);
  }
  return (sortFlags & LF_SORT_REVERSE) ? -res : res;
}

char GetFileChar(int attr) {
  if (attr & 0x08) {
    /* Label */
    return 'L';
  } else if (attr & 0x10) {
    /* Directory */
    return 'D';
  }
  /* File */
  return '-';
}

struct AudioInfo audioInfo;

s_int16 ListFiles(const char *path, int flags, int __mem_y *idx) {
  s_int16 n, nMedia;
  static char filename[NAMELENGTH];
  int fileIndicesLeft = *idx; //Only valid if idx is non-NULL
  FILE *f = fopen(path,"s"); //Get search handle for the disk

  if (f) {
    path += 2; //Remove drive letter and colon from path
  doItAgain:
    n = 0;
    nMedia = 0;
    if (FatFindFirst(f,path,filename,NAMELENGTH) == S_OK) {
#if 0
      printf("Entries in %s \"%s\", in %s order:\n",
	     f->dev->Identify(f->dev,NULL,0), path-2,
	     sorted ? "sorted" : "physical");
#endif
      do {
	n++;
	if (!(flags & LF_AUDIO_ONLY) ||
	    !GetAudioInfo(filename, &audioInfo, NULL, GAIF_FAST)) {
	  if (flags & LF_SORT_MASK) {
	    struct SortDirEntry *sde =
	      calloc(1, sizeof(struct SortDirEntry) + strlen(filename));
	    if (!sde || n >= SORT_ELEMENTS-1) {
	      FreeAllEntries();
	      flags &= ~LF_SORT_MASK;
	      goto doItAgain;
	    }
	    sde->idx = n;
	    sde->attr = f->ungetc_buffer;
	    sde->size = FoundFileSize(f);
	    {
	      u_int32 t = FoundFileTimeDate(f);
	      sde->time = (u_int16)t;
	      sde->date = (u_int16)(t>>16);
	    }
	    strcpy(sde->shortName, f->extraInfo);
	    strcpy(sde->longName, filename);
	    sortDirEntry[nMedia] = sde;
	    nMedia++;
	  } else {
	    if (idx) {
	      if (fileIndicesLeft > 0) {
		fileIndicesLeft--;
		*idx++ = n;
	      }
	    } else {
	      if (flags & LF_AUDIO_ONLY) {
		int aiStatus;
		int minutes;
		memset(&audioInfo, 0, sizeof(audioInfo));
		aiStatus = GetAudioInfo(filename, &audioInfo, NULL,
					(flags & LF_FAST) ? GAIF_FAST : 0);

		minutes = audioInfo.seconds/60.0;
		audioInfo.seconds -= minutes*60.0;
		printf("%c %4d. %-12s %4d:%04.1f %6ld %1d",
		       GetFileChar(f->ungetc_buffer), n, f->extraInfo,
		       minutes, audioInfo.seconds,
		       audioInfo.sampleRate, audioInfo.channels);
	      } else {
		printf("%c %4d. %-12s %10lu",
		       GetFileChar(f->ungetc_buffer), n, f->extraInfo,
		       FoundFileSize(f));
	      }
	      if (flags & LF_SHOWDATE) {
		static u_int16 Y,M,D,h,m,s,t;
		t = (u_int16)(FoundFileTimeDate(f)>>16);
		D = t & 31;
		t >>= 5;
		M = t & 15;
		Y = (1980+(t>>4));
		t = (u_int16)FoundFileTimeDate(f);
		s = (t & 31) * 2;
		t >>= 5;
		m = t & 63;
		h = t>>6;
		printf(" %02d-%02d-%02d %02d:%02d:%02d",
		       Y, M, D, h, m, s);
	      }
	      printf(" %s\n", filename);
	    }
	    nMedia++;
	  }
	}
      } while (S_OK == FatFindNext(f,filename,255));
    } else {
      printf("Path not found\n");
    }
    fclose(f);

    if (flags & LF_SORT_MASK) {
      int i;
      sortFlags = flags;
      qsort(sortDirEntry, nMedia, sizeof(sortDirEntry[0]), SortDirEntryCmp);
      for (i=0; i<nMedia; i++) {
	struct SortDirEntry *sde = sortDirEntry[i];

	if (idx) {
	  if (fileIndicesLeft > 0) {
	    fileIndicesLeft--;
	    *idx++ = sde->idx;
	  }
	} else {
	  if (flags & LF_AUDIO_ONLY) {
	    int aiStatus;
	    int minutes;
	    memset(&audioInfo, 0, sizeof(audioInfo));
	    aiStatus = GetAudioInfo(sde->longName, &audioInfo, NULL,
				    (flags & LF_FAST) ? GAIF_FAST : 0);
	    minutes = audioInfo.seconds/60.0;
	    audioInfo.seconds -= minutes*60.0;
	    printf("%c %4d. %-12s %4d:%04.1f %6ld %1d",
		   GetFileChar(sde->attr), sde->idx, sde->shortName,
		   minutes, audioInfo.seconds,
		   audioInfo.sampleRate, audioInfo.channels);
	  } else {
	    printf("%c %4d. %-12s %10lu",
		   GetFileChar(sde->attr), sde->idx, sde->shortName, sde->size);
	  }
	  if (flags & LF_SHOWDATE) {
	    static u_int16 Y,M,D,h,m,s,t;
	    t = sde->date;
	    D = t & 31;
	    t >>= 5;
	    M = t & 15;
	    Y = (1980+(t>>4));
	    t = sde->time;
	    s = (t & 31) * 2;
	    t >>= 5;
	    m = t & 63;
	    h = t>>6;
	    printf(" %02d-%02d-%02d %02d:%02d:%02d",
		   Y, M, D, h, m, s);
	  }
	  printf(" %s\n", sde->longName);
	}
      }
      FreeAllEntries();
    }
  }

  return nMedia;
}


ioresult main(char *parameters) {
  int nParam, nDirParam = 0, listFlags = LF_SORT_BY_NAME|LF_FAST|LF_SHOWDATE, i;

  nParam = RunLibraryFunction("ParamSpl",ENTRY_MAIN,(u_int16)parameters);

  for (i=0; i<nParam; i++) {
    if (!strcmp(parameters, "-h")) {
      printf("Usage: dir [-s|-sn|-st|-ss|+s|-r|+r|-d|+d|-a|+a|-f|+f|-h] [fullpath]\n"
	     "-s|-sn\tSort files by name\n"
	     "-st\tSort files by date\n"
	     "-ss\tSort files by size\n"
	     "+s\tDon't sort files (faster with large directories)\n"
	     "-r\tReverse sort (if sort selected)\n"
	     "+r\tForwards sort (if sort selected)\n"
	     "-d\tShow file date\n"
	     "+d\tDon't show file date\n"
	     "-a\tOnly audio files\n"
	     "+a\tAll files\n"
	     "-f\tFast listing (don't show play time for MP3 files with -a)\n"
	     "+f\tSlower listing (show play time also for MP3 files with -a)\n"
	     "-h\tShow this help page\n");
      goto finally;
    } else if (!strcmp(parameters, "-a")) {
      listFlags |= LF_AUDIO_ONLY;
    } else if (!strcmp(parameters, "+a")) {
      listFlags &= ~LF_AUDIO_ONLY;
    } else if (!strcmp(parameters, "-s") || !strcmp(parameters, "-sn")) {
      listFlags &= ~LF_SORT_MASK;
      listFlags |= LF_SORT_BY_NAME;
    } else if (!strcmp(parameters, "-st")) {
      listFlags &= ~LF_SORT_MASK;
      listFlags |= LF_SORT_BY_TIME;
    } else if (!strcmp(parameters, "-ss")) {
      listFlags &= ~LF_SORT_MASK;
      listFlags |= LF_SORT_BY_SIZE;
    } else if (!strcmp(parameters, "+s")) {
      listFlags &= ~LF_SORT_MASK;
    } else if (!strcmp(parameters, "-r")) {
      listFlags |= LF_SORT_REVERSE;
    } else if (!strcmp(parameters, "+r")) {
      listFlags &= ~LF_SORT_REVERSE;
    } else if (!strcmp(parameters, "-d")) {
      listFlags |= LF_SHOWDATE;
    } else if (!strcmp(parameters, "+d")) {
      listFlags &= ~LF_SHOWDATE;
    } else if (!strcmp(parameters, "-f")) {
      listFlags |= LF_FAST;
    } else if (!strcmp(parameters, "+f")) {
      listFlags &= ~LF_FAST;
    } else {
      char e = parameters[strlen(parameters)-1];
      if ((e != ':') && (e != '/')) {
	printf("'%s' is not a valid path (did you mean '%s/'?)\n",
	       parameters, parameters);
	return S_ERROR;
      }
      ListFiles(parameters, listFlags, NULL);
      nDirParam++;
    }
    parameters += strlen(parameters)+1;
  }

  if (!nDirParam) {
    ListFiles(currentDirectory, listFlags, NULL);
  }

 finally:
  return S_OK;
}



DLLENTRY(MakePlayList)
ioresult MakePlayList(u_int16 __mem_y *idx) {
  return ListFiles(currentDirectory, LF_SORT_BY_NAME|LF_FAST|LF_AUDIO_ONLY, idx);
}
