/*
  Ogg Vorbis Encoder library ENCVORB.DL3 v1.00
  VLSI Solution 2014-10-09 HH

  This library creates Ogg Vorbis files from a PCM input.

  To use this device driver, copy the file ENCVORB.DL3 to the SYS
  folder of your VS1005 boot device.

  For further details on how to use this library, see the
  Simple MP3 Encoder application.

  This code may be used freely in any product containing one or more ICs
  by VLSI Solution.

  No guarantee is given for the usability of this code.

 */

/* For free support for VSIDE, please visit www.vsdsp-forum.com */

// Starting point template for creating VSOS applications.
// This will create a <projectname>.AP3 file, which you can copy to 
// your VS1005 Developer Board and run it from there.

// If you rename your application to INIT.APP, then the kernel will
// load it automatically after booting.

#include <vo_stdio.h>
#include <volink.h>     // Linker directives like DLLENTRY
#include <apploader.h>  // RunLibraryFunction etc
#include <sysmemory.h>
#include "encodervorbis.h"
#include "encVCreate.h"


struct Encoder *enc = NULL;
void (*Delete)(struct Encoder *enc) = NULL;
void fini(void);


DLLENTRY(Create)
struct Encoder *Create(register struct EncoderServices *es,
		       register u_int16 channels, register u_int32 sampleRate,
		       register u_int16 quality) {
	if (enc = EncVorbisCreate(es, channels, sampleRate, quality)) {
		// Remember original Delete() function address
	  Delete = enc->Delete;
	  // Grab Delete() so that we can do our own clean-up
	  enc->Delete = (void *)fini;
	}
	return enc;
}

void fini(void) {
	if (enc) {
		if (Delete) {
			Delete(enc);
			Delete = NULL;
		}
		enc = NULL;
	}
	UnloadEncV();
}



#define MAX_WINF_BITS 11
#define MAX_WINF_SIZE (1<<MAX_WINF_BITS)

DLLENTRY(preloadResult)
u_int16 preloadResult;

int preload(void) {
	return AllocMemXY(2*MAX_WINF_SIZE, MAX_WINF_SIZE); 
}
