/* For free support for VSIDE, please visit www.vsdsp-forum.com */

/// \file devramdisk.c VsOS Device driver for ram disk
/// \author Panu-Kristian Poiksalo, VLSI Solution Oy

#include <vo_stdio.h>
#include <volink.h>     // Linker directives like DLLENTRY
#include <apploader.h>  // RunLibraryFunction etc
#include <stdlib.h>
#include <string.h>
#include <timers.h>
#include <clockspeed.h>
#include <vsos.h>


#define RAMDISK_SIZE_BLOCKS 36 //fixed format: 1 block for BPB, 1 block for FAT, 1 block for root, 16.5KB remains.

__y u_int16 ram[RAMDISK_SIZE_BLOCKS][256] = {
	/* 00-01 */ 0xeb34,
	/* 02-03 */ 0x9000|'M',
	/* 04-05 */ ('S'<<8)|'D',
	/* 06-07 */ ('O'<<8)|'S',
	/* 08-09 */ ('4'<<8)|'.',
	/* 10-11 */ '0'<<8|0x00,
	/* 11-13 */ 0x0201, //bytespersector | sectorspercluster
	/* 14-15 */ 0x0100, //reserved sectors
	/* 16-17 */ 0x0110, //numfats | nrootentlo
	/* 18-19 */ 0x0024, //nrootenthi | totsec16lo
	/* 20-21 */ 0x00f8, //totsec16hi|media
	/* 22-23 */ 0x0100, //fat size
	/* 24-25 */ 0x2400, //sectors per track;
	/* 26-27 */ 0x0100, //number of tracks;
	/* 28-29 */ 0x0000, //HiddSec32
	/* 30-31 */ 0x0000, //Hiddsec32
	/* 32-33 */ 0x0000, //0x0a00; //totsec32;
	/* 34-35 */ 0x0000, //totsec32;
	/* 36-37 */ 0x0000, //logicaldrivenum | reserved
	/* 38-39 */ 0x2801, //extendedsig | sernum
	/* 40-41 */ 0x0203, //
	/* 42-43 */ 0x0400|'V',
	/* 44-45 */ 'L'<<8|'S',
	/* 46-47 */ 'I'<<8|'F',
	/* 48-49 */ 'A'<<8|'T',
	/* 50-51 */ 'D'<<8|'I',
	/* 52-53 */ 'S'<<8|'K',
	/* 54-55 */ 'F'<<8|'A',
	/* 56-57 */ 'T'<<8|'1',
	/* 58-59 */ '2'<<8|' ',
	/* 60-61 */ ' '<<8|' '
};


typedef struct DevRamDiskHardwareInfo {
	u_int16 totalBlocks;
} devRamDiskHwInfo;


ioresult DevRamDiskBlockRead(register __i0 DEVICE *dev, u_int32 firstBlock, u_int16 blocks, u_int16 *data);
ioresult DevRamDiskBlockWrite(register __i0 DEVICE *dev, u_int32 firstBlock, u_int16 blocks, u_int16 *data);
char* DevRamDiskIdentify(register __i0 void *obj, char *buf, u_int16 bufsize);
IOCTL_RESULT DevRamDiskIoctl(register __i0 DEVICE *dev, s_int16 request, IOCTL_ARGUMENT arg);


DEVICE ramdisk = {   
  0, // u_int16 flags; //< present, block/char
  DevRamDiskIdentify, //char*    (*Identify)(void *obj, char *buf, u_int16 bufsize);   
  CommonOkResultFunction, //ioresult (*Create)(DEVICE *dev, char *name, struct filesystem_descriptor *fs);
  CommonOkResultFunction, //ioresult (*Delete)(DEVICE *dev);
  DevRamDiskIoctl, //ioresult (*Ioctl)(DEVICE *dev, s_int16 request, s_int32 arg); //Start, Stop, Flush, Check Media
  // Stream operations
  0, //u_int16  (*Read) (register __i0 DEVICE *self, void *buf, u_int16 destinationIndex, u_int16 bytes); ///< Read bytes to *buf, at byte index
  0, //u_int16  (*Write)(register __i0 DEVICE *self, void *buf, u_int16 sourceIndex, u_int16 bytes); ///< Write bytes from *buf at byte index
  // Block device operations
  DevRamDiskBlockRead, //ioresult (*BlockRead)(DEVICE *dev, u_int32 firstBlock, u_int16 blocks, u_int16 *data);
  DevRamDiskBlockWrite, //ioresult (*BlockWrite)(DEVICE *dev, u_int32 firstBlock, u_int16 blocks, u_int16 *data);
  // Stream operations
  //DevRamDiskInput, //ioresult (*Input)(DEVICE *dev, u_int16 port, void *buf, u_int16 bytes);
  //DevRamDiskOutput, //ioresult (*Output)(DEVICE *dev, u_int16 port, void *buf, u_int16 bytes);
  //FILESYSTEM *fs;
  //DIRECTORY *root; //Zero if the device cannot hold directories
  //u_int16  deviceInstance;
  //u_int16  hardwareInfo[6]; ///< Device driver's info of this hardware
  //u_int16  deviceInfo[10]; // For filesystem use, size TBD   
};
void BlockRead(register __i0 DEVICE *phy, register u_int32 firstBlock, u_int16 *data);
 
#define END_FRAME 1
#define DONT_END_FRAME 0
  

const char *DevRamDiskIdentify(register __i0 void *dev, char *buf, u_int16 bufsize) {
	return "Ramdisk";
}


ioresult DevRamDiskBlockRead(register __i0 DEVICE *dev, u_int32 firstBlock, u_int16 blocks, u_int16 *data){
	devRamDiskHwInfo *hw = &(dev->hardwareInfo);
	//printf(" r%ld",firstBlock);
	while(blocks--) {
		memcpyYX(data, ram[firstBlock], 256);
		firstBlock++;
		data += 256;
	}
	return S_OK;
}
ioresult DevRamDiskBlockWrite(register __i0 DEVICE *dev,  u_int32 firstBlock, u_int16 blocks, u_int16 *data){
	devRamDiskHwInfo *hw = &(dev->hardwareInfo);
	while(blocks--) {
		//printf("\n #W%ld# ",firstBlock);
		memcpyXY(ram[firstBlock], data, 256);
		firstBlock++;
		data += 256;
	}
	return S_OK;
}


IOCTL_RESULT DevRamDiskIoctl(register __i0 DEVICE *dev, s_int16 request, IOCTL_ARGUMENT arg) {
	ioresult errCode = S_OK;
	devRamDiskHwInfo *hw = &(dev->hardwareInfo);
	u_int16 i, n, cmd;
	switch (request) {
		case IOCTL_RESTART: {
			dev->fs = StartFileSystem(dev, "0");
			if (!dev->fs) {
				dev->flags = __MASK_PRESENT;
				return S_ERROR;
			}
			break;
		}
		case IOCTL_GET_GEOMETRY:
			if (arg) {
				((DiskGeometry*)arg)->sectorsPerBlock = 1;
				((DiskGeometry*)arg)->totalSectors = hw->totalBlocks;
			}
			return hw->totalBlocks; //obsolete
			break;
		default: {
			errCode = S_ERROR;
			break;
		}
	}
	return errCode;
}


ioresult main(char *parameters) {
	ramdisk.hardwareInfo[0] = RAMDISK_SIZE_BLOCKS;
	ram[2][0] = ('R'<<8) | 'A';
	ram[2][1] = ('M'<<8);
	ramdisk.Ioctl(&ramdisk, IOCTL_RESTART, NULL);
	vo_pdevices[(0x20|parameters[0])-'a'] = &ramdisk;	
	return S_OK;
}

